#include "BehaviorEditor.h"

NodePresent::NodePresent(std::shared_ptr<LogicalNode> bind)
	:_node_bind(bind) {
}

QRectF NodePresent::contentMeasure() const
{
	return QRectF(0, 0, 200, 40); 
}

QRectF NodePresent::boundingRect() const
{
	return contentMeasure() + QMargins(0, 0, 20, 20);
}

void NodePresent::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	//painter->drawRect(boundingRect());
	painter->translate(QPointF(10, 10));

	auto rect = contentMeasure();
	painter->fillRect(rect, Qt::red);
}

BehaviorsPresent::BehaviorsPresent(QWidget* parent /*= nullptr*/)
	: QGraphicsView(parent) {
	this->setScene(&_bind_scene);
}

void BehaviorsPresent::setRoot(std::shared_ptr<BehaviorMapNode> root)
{
	qDeleteAll(_present_peers);
	_present_peers.clear();

	this->_bind_maproot = root;
	contentRelayout();
}

void BehaviorsPresent::presentAllocate(std::shared_ptr<LogicalNode> ins)
{
	switch (ins->nodeKind())
	{
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SEQUENCENODE:
	case NodeKind::SELECTORNODE:
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			auto type = ins->nodeKind();
			for (auto it : ins->children())
				presentAllocate(it);
		}

	default:
		if (_present_peers.contains(ins))
			return;
		
		_present_peers[ins] = new NodePresent(ins);
		this->_bind_scene.addItem(_present_peers[ins]);
		break;
	}
}

void BehaviorsPresent::presentRelease(std::shared_ptr<LogicalNode> ins)
{
	switch (ins->nodeKind())
	{
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SELECTORNODE:
	case NodeKind::SEQUENCENODE: {
		for (auto item : ins->children()) {
			presentRelease(item);
		}
	}break;
	default:
		if (this->_present_peers.contains(ins))
			delete this->_present_peers[ins];
		break;
	}
}

void BehaviorsPresent::contentRelayout()
{
	if (!_bind_maproot)
		return;

	// ʼʾڵ
	presentAllocate(_bind_maproot);

	QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>> results;
	// ߴ
	outlineMeasure(_bind_maproot, results);
	// ԪŲ
	nodeRelayout(results, _bind_maproot, QPointF());
}

QSizeF BehaviorsPresent::outlineMeasure(std::shared_ptr<LogicalNode> ins, QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy)
{
	QSizeF outline_box(0, 0);

	switch (ins->nodeKind()) {
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SELECTORNODE:
	case NodeKind::SEQUENCENODE:
		// ӽڵ㼯ϳߴ磬ڵΪڵ
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			for (auto nit : ins->children()) {
				auto nit_size = outlineMeasure(nit, _outline_occupy);
				outline_box.setWidth(std::max(outline_box.width(), nit_size.width() + _space_h));
				outline_box += QSizeF(0, nit_size.height());
			}
		}
	default: {
		auto root_peer = _present_peers[ins];
		auto root_box = root_peer->boundingRect();
		outline_box += QSizeF(root_box.width(), 0);
		outline_box.setHeight(std::max(outline_box.height(), root_box.height()));

		_outline_occupy[ins] = std::make_pair(outline_box, root_box.size());
		return outline_box;
	}break;
	}
}

QPointF BehaviorsPresent::nodeRelayout(QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy,
	std::shared_ptr<LogicalNode> ins, const QPointF& origin_offset) {
	// ȡߴ¼
	auto outline_occupy = _outline_occupy[ins];
	auto node_outline = outline_occupy.first;
	auto node_occupy = outline_occupy.second;

	// ȡʾڵ
	auto present_node = _present_peers[ins];

	// ýڵλ
	QPointF local_origin_offset(0, (node_outline.height() - node_occupy.height()) / 2);
	present_node->setPos(origin_offset + local_origin_offset);

	// ӽڵλ
	QPointF child_origin = origin_offset + QPointF(node_occupy.width() + _space_h, 0);
	switch (ins->nodeKind())
	{
	case NodeKind::ACTIONNODE:
	case NodeKind::COMPARENODE:
		break;
	default:
		// ӽڵ㼯λãڵǶ˸ڵ
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			for (auto item : ins->children()) {
				child_origin = nodeRelayout(_outline_occupy, item, child_origin);
			}
		}break;
	}

	// ½λ
	return origin_offset + QPointF(0, node_outline.height());
}

uint qHash(const std::shared_ptr<LogicalNode> data, uint seed) noexcept
{
	return qHash((void*)data.get(), seed);
}
